@@ -16,8 +16,8 @@
 
 import yaml
 
+from tower_cli import exceptions as exc
 from tower_cli.utils import parser
-from tower_cli.utils import exceptions as exc
 from tower_cli.utils.data_structures import OrderedDict
 
 from tests.compat import unittest, mock
@@ -147,7 +147,7 @@ class ParserTests(unittest.TestCase):
 
 
 class TestSplitter_Gen(unittest.TestCase):
-    """Set of strings paired with expected output is ran agains the parsing
+    """Set of strings paired with expected output is ran against the parsing
     functions in this code in order to verrify desired accuracy.
 
     SPLIT_DATA is taken from Ansible tests located in:
