@@ -15,10 +15,11 @@
 # limitations under the License.
 
 from tower_cli import models
-from tower_cli.utils import types
+from tower_cli.cli import types
 
 
 class Resource(models.Resource):
+    """A resource for inventory scripts."""
     cli_help = 'Manage inventory scripts within Ansible Tower.'
     endpoint = '/inventory_scripts/'
 
